//--------------------------------------------------------------------
// >>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<<<
//--------------------------------------------------------------------
//////////////////////////////////////////////////////////////////////
//
//-------------------------------------------------------------------
//--                                                             ----
//--  OpenCores IDE Controller                                   ----
//--  DMA (single- and multiword) mode timing statemachine       ----
//--                                                             ----
//--  Author: Richard Herveille                                  ----
//--          richard@asics.ws                                   ----
//--          www.asics.ws                                       ----
//--                                                             ----
//-------------------------------------------------------------------
//--                                                             ----
//-- Copyright (C) 2001, 2002 Richard Herveille                  ----
//--                          richard@asics.ws                   ----
//--                                                             ----
//-- This source file may be used and distributed without        ----
//-- restriction provided that this copyright statement is not   ----
//-- removed from the file and that any derivative work contains ----
//-- the original copyright notice and the associated disclaimer.----
//--                                                             ----
//--     THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY     ----
//-- EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED   ----
//-- TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS   ----
//-- FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE AUTHOR      ----
//-- OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,         ----
//-- INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES    ----
//-- (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE   ----
//-- GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR        ----
//-- BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF  ----
//-- LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT  ----
//-- (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  ----
//-- OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE         ----
//-- POSSIBILITY OF SUCH DAMAGE.                                 ----
//--                                                             ----
//-------------------------------------------------------------------
// rev.: 1.0 march 7th, 2001. Initial release
//
//  CVS Log
//
//  $Id: atahost_dma_tctrl.vhd,v 1.1 2002/02/18 14:32:12 rherveille Exp $
//
//  $Date: 2002/02/18 14:32:12 $
//  $Revision: 1.1 $
//  $Author: rherveille $
//  $Locker:  $
//  $State: Exp $
//
// Change History:
//               $Log: atahost_dma_tctrl.vhd,v $
//               Revision 1.1  2002/02/18 14:32:12  rherveille
//               renamed all files to 'atahost_***.vhd'
//               broke-up 'counter.vhd' into 'ud_cnt.vhd' and 'ro_cnt.vhd'
//               changed resD input to generic RESD in ud_cnt.vhd
//               changed ID input to generic ID in ro_cnt.vhd
//               changed core to reflect changes in ro_cnt.vhd
//               removed references to 'count' library
//               changed IO names
//               added disclaimer
//               added CVS log
//               moved registers and wishbone signals into 'atahost_wb_slave.vhd'
//
//
//
//-------------------------
// DMA Timing Controller --
//-------------------------
//
//
// Timing	DMA mode transfers
//--------------------------------------------
// T0:	cycle time
// Td:	DIOR-/DIOW- asserted pulse width
// Te: DIOR- data access
// Tf: DIOR- data hold
// Tg: DIOR-/DIOW- data setup
// Th: DIOW- data hold
// Ti: DMACK to DIOR-/DIOW- setup
// Tj: DIOR-/DIOW- to DMACK hold
// Tkr: DIOR- negated pulse width
// Tkw: DIOW- negated pulse width
// Tm: CS(1:0) valid to DIOR-/DIOW-
// Tn: CS(1:0) hold
//
//
// Transfer sequence
//--------------------------------
// 1) wait for Tm
// 2) assert DIOR-/DIOW-
//    when write action present data (Timing spec. Tg always honored)
//    output enable is controlled by DMA-direction and DMACK-
// 3) wait for Td
// 4) negate DIOR-/DIOW-
//    when read action, latch data
// 5) wait for Teoc (T0 - Td - Tm) or Tkw, whichever is greater
//    Th, Tj, Tk, Tn always honored
// 6) start new cycle
//
//---------------------------------------------------------------------
// Copyright (c) 2010 by Lattice Semiconductor Corporation
//---------------------------------------------------------------------
// Disclaimer:
//
// This VHDL or Verilog source code is intended as a design reference
// which illustrates how these types of functions can be implemented.
// It is the user's responsibility to verify their design for
// consistency and functionality through the use of formal
// verification methods. Lattice Semiconductor provides no warranty
// regarding the use or functionality of this code.
//
// --------------------------------------------------------------------
//
// Lattice Semiconductor Corporation
// 5555 NE Moore Court
// Hillsboro, OR 97214
// U.S.A
//
// TEL: 1-800-Lattice (USA and Canada)
// 503-268-8001 (other locations)
//
// web: http://www.latticesemi.com/
// email: techsupport@latticesemi.com
//
// --------------------------------------------------------------------
// Code Revision History :
// --------------------------------------------------------------------
// Ver: | Author     |Mod. Date |Changes Made:
// V1.0 | Peter.Zhou |06/02/10  |initial release
//
// --------------------------------------------------------------------
`timescale 1ns/1ns
module atahost_dma_tctrl (clk, nReset, rst, Tm, Td, Teoc, go, we, done, dstrb, DIOR, DIOW);

    // counter width
   parameter TWIDTH = 8;
    // 50ns
   parameter DMA_mode0_Tm = 4;
    // 215ns
   parameter DMA_mode0_Td = 21;
   parameter DMA_mode0_Teoc  = 21;
   input clk; // master clock
   input nReset; // asynchronous active low reset
   input rst; // synchronous active high reset
   input[TWIDTH - 1:0] Tm; // Tm time (in clk-ticks)
   input[TWIDTH - 1:0] Td; // Td time (in clk-ticks)
   input[TWIDTH - 1:0] Teoc; // end of cycle time
   input go; // DMA controller selected (strobe signal)
   input we; // DMA direction '1' = write, '0' = read
   output done; // finished cycle
   wire done;
   output dstrb; // data strobe
   reg dstrb;
   output DIOR; 
   reg DIOR;
   output DIOW; 
   reg DIOW;

   // initial data after reset
   // current counter value
   wire Tmdone; 
   wire Tddone; 

   // 1)	hookup Tm counter

  
   ro_cnt #(TWIDTH,     
						1'b0,        
					DMA_mode0_Tm
) tm_cnt(
      .clk(clk), 
      .nReset(nReset), 
      .rst(rst), 
      .go(go), 
      .d(Tm), 
      .cnt_en(1'b1),
      .q(),
      .done(Tmdone)
   ); 

   // 2)	set (and reset) DIOR-/DIOW-
   always @(posedge clk or negedge nReset)
   begin : T2proc
      if (nReset == 1'b0)
      begin
         DIOR <= 1'b0 ; 
         DIOW <= 1'b0 ; 
      end
      else
      begin
         if (rst == 1'b1)
         begin
            DIOR <= 1'b0 ; 
            DIOW <= 1'b0 ; 
         end
         else
         begin
            DIOR <= (~we & Tmdone) | (DIOR & ~Tddone) ; 
            DIOW <= (we & Tmdone) | (DIOW & ~Tddone) ; 
         end 
      end 
   end 
   // 3)	hookup Td counter


   ro_cnt #(TWIDTH,      
					1'b0,         
					DMA_mode0_Td 
				) td_cnt(
      .clk(clk), 
      .nReset(nReset), 
      .rst(rst), 
      .go(Tmdone), 
      .d(Td), 
      .cnt_en(1'b1),
      .q(),
      .done(Tddone)
   ); 

   // generate data_strobe
   always @(posedge clk)
   begin : gen_dstrb
      dstrb <= Tddone ; // capture data at rising edge of DIOR- 
   end 
   // 4) negate DIOR-/DIOW- when Tddone
   // 5)	hookup end_of_cycle counter


   ro_cnt #(TWIDTH,      
					1'b0,         
					DMA_mode0_Teoc ) eoc_cnt(
      .clk(clk), 
      .nReset(nReset), 
      .rst(rst), 
      .go(Tddone), 
      .d(Teoc), 
      .cnt_en(1'b1),
      .q(),
      .done(done)
   ); 
endmodule
